import React, { Component, ChangeEvent } from 'react';


/** Represents an item in a Todo list. */
type TodoItem = {
  /** Name of the todo item. */
  name: string;

  /** Indicates if the todo item has been completed. */
  completed: boolean;
};

type TodoProps = {
  /** Value of the items query parameter. */
  initialItems: TodoItem[]
};

type TodoState = {
  /** Current TodoItems */
  items: TodoItem[]
};

/** Displays the todo UI. */
export class Todo extends Component<TodoProps, TodoState> {
  constructor(props: TodoProps) {
    super(props);

    this.state = {items: props.initialItems.slice(0)};
  }

  render = (): JSX.Element => {
    return (
      <div>
        <h2>To-Do List</h2>
        {this.renderItems()}
      </div>);
  };

  // Returns a div for each item in the todo list, with each div containing a
  // label with the name of the item and a checkbox for marking it completed.
  renderItems = (): JSX.Element[] => {
    const items: JSX.Element[] = [];
    // Inv: items contains a div for each item of this.state.items[0 .. i-1]
    for (const item of this.state.items) {
      const i = this.state.items.findIndex((i) => i === item);
      const id = "check_" + i;
      if (item.completed) {
        items.push(
          <div key={i}>
            <input type="checkbox" id={id} checked={true} readOnly={true} />
            <label className="completed" htmlFor={id}>{item.name}</label>
          </div>);
      } else {
        items.push(
          <div key={i}>
            <input type="checkbox" id={id} checked={false}
                onChange={(e) => this.doItemClick(e, i)}/>
            <label htmlFor={id}>{item.name}</label>
          </div>);
      }
    }
    return items;
  };

  // Called when the user checks the box next to an uncompleted item. index is
  // the index of that item in the list.
  doItemClick = (_: ChangeEvent<HTMLInputElement>, index: number): void => {
    // Note: we cannot mutate the list. We have to create a new one.
    const newItems = this.state.items.slice(0, index)
        .concat(this.state.items.slice(index + 1));

    this.setState({items: newItems});
  };
}
