import React from 'react';
import { createRoot, Root } from 'react-dom/client';
import { Todo } from './Todo';


const params: URLSearchParams = new URLSearchParams(window.location.search);
const itemList: string | null = params.get("items");

const main: HTMLElement | null = document.getElementById('main');
if (main === null)
  throw new Error("HTML missing 'main' element");

const root: Root = createRoot(main);
if (itemList === null) {
  root.render(<p>Error: missing "items" parameter</p>);
} else {
  // Splits itemList query params string into a list of items
  // NOTE: Only simple function expressions are allowed, the linter does
  // not allow the following code. We are using this code here as an
  // intermediate step. The user shouldn't have to pass in a list via
  // query params, which is why the query params are replaced with a
  // text box in Todo v2. THIS IS A PLACEHOLDER!!! :)
  const items = itemList.split(",")
      .filter(s => s.trim().length > 0)
      .map(s => { return {name: s.trim(), completed: false}; });
  root.render(<Todo initialItems={items}/>);
}
